Attribute VB_Name = "modAverageNonZero"
Option Explicit

'>>>>>>>>> BEGIN CODE
Function AverageNonZero(FirstSheet As String, _
        LastSheet As String, _
        InDataRange As Range) As Variant
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' AverageNonZero
' By Chip Pearson, chip@cpearson.com www.cpearson.com
' This averages all non-zero cells on worksheets from FirstSheet
' to LastSheet in the range specified by InDataRange. InDataRange
' can reference a range on any worksheet -- the code uses the
' Address of the range to create the actual ranges on each sheet.
' Empty cells are assumed to have a value equal to zero thus are
' not included in the result. The boolean values TRUE and FALSE
' are not included in the result.
'
' This function returns #REF if FirstSheet does not exist, if
' LastSheet does not exist, if LastSheet is to the left of
' FirstSheet, or InDataRange is Nothing. If no non-zero values
' are found, the result is #DIV/0!.
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Dim WSStart As Long
Dim WSEnd As Long
Dim WSNdx As Long
Dim Addr As String
Dim R As Range
Dim Count As Long
Dim SumValues As Double
Dim SheetDataRange As Range

With ThisWorkbook.Worksheets
On Error Resume Next
Err.Clear
' Do some error checking.
WSStart = .Item(FirstSheet).Index
If Err.Number <> 0 Then
    AverageNonZero = CVErr(xlErrRef)
    Exit Function
End If
WSEnd = .Item(LastSheet).Index
If Err.Number <> 0 Then
    AverageNonZero = CVErr(xlErrRef)
    Exit Function
End If
If WSStart > WSEnd Then
    AverageNonZero = CVErr(xlErrRef)
    Exit Function
End If
If InDataRange Is Nothing Then
    AverageNonZero = CVErr(xlErrRef)
    Exit Function
End If

' Addr is the address of the range on each sheet to average.
Addr = InDataRange.Address
For WSNdx = WSStart To WSEnd
    ' Use Addr to get a range on Workshets(WSNdx).
    Set SheetDataRange = .Item(WSNdx).Range(Addr)
    For Each R In SheetDataRange.Cells
        If Len(R.Value) > 0 Then
            If IsNumeric(R.Text) Then
                If R.Value <> 0 Then
                    ' You can modify the line of code above to
                    ' include only certain values in the average.
                    ' This example averages only non-zero values,
                    ' but you can create any test you want.
                
                    ' Non-zero value found. Increment Count
                    ' and add to SumValues.
                    Count = Count + 1
                    SumValues = SumValues + R.Value
                End If
            End If
        End If
    Next R
Next WSNdx
End With
If Count = 0 Then
    ' No non-zero elements found.
    AverageNonZero = CVErr(xlErrDiv0)
    Exit Function
End If
' Average is SumValues / Count.
AverageNonZero = SumValues / Count
End Function
'>>>>>>>>> END CODE
